#!/usr/bin/python3
"""
Data used for communication protocol between cu and master nodes
"""
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################
from datetime import datetime
from enum import Enum

#######################       THIRD PARTY IMPORTS        #######################

#######################    SYSTEM ABSTRACTION IMPORTS    #######################

#######################       LOGGER CONFIGURATION       #######################

#######################          MODULE IMPORTS          #######################

#######################          PROJECT IMPORTS         #######################

#######################              ENUMS               #######################
class CommDataRegisterTypeE(Enum):
    '''
    Define the message types allowed on protocol used to discover the cu_id of each node
    '''
    DISCOVER = 0
    OFFER = 1
    REQUEST = 2
    ACK = 3


class CommDataMnCmdTypeE(Enum):
    '''
    Define the message types allowed on protocol used to send commands among services of master node
    '''
    LAUNCH = 1
    INF_DEV = 2
    REQ_DETECT = 3


######################             CLASSES              #######################
class CommDataCuC:
    '''
    Class used to store data of a computational unit (CU).
    '''

    def __init__(self, msg_type : CommDataRegisterTypeE, mac : int, user : str,
            ip : str, port : int, hostname : str, cu_id : int = 0):
        # pylint: disable=too-many-arguments
        # pylint: disable=invalid-name
        '''
        Initialize the class with the computational unit (CU) data.

        Args:
            msg_type (CommDataRegisterTypeE): message type
            mac (int): mac of the CU
            user (str): user of the CU
            ip (str): ip of the CU
            port (int): not used
            hostname (str): hostname of the CU
            cu_id (int, optional): CU ID assigned to this CU. Defaults to None.
        '''
        self.msg_type = msg_type
        self.mac = mac
        self.user = user
        self.ip = ip # pylint: disable=invalid-name
        self.port = port
        self.hostname = hostname
        self.cu_id = cu_id

    def __str__(self) -> str:
        '''
        Return a string with the data of the CU.

        Returns:
            str: string with the data of the CU.
        '''
        return f'CU info: \nCU_ID: {self.cu_id}\nMAC: {self.mac}\nUser: {self.user}\n' + \
            f'Hostname: {self.hostname}\nIP: {self.ip}\nPort: {self.port}\nMsgType: {self.msg_type}'


class CommDataHeartbeatC:
    '''
    Class used to store data related with a heartbeat.
    '''

    def __init__(self, cu_id : int):
        '''
        Initialize the class with the heartbeat info.

        Args:
            cu_id (int): [description]
        '''
        self.cu_id = cu_id
        self.timestamp = datetime.utcnow()

    def __str__(self) -> str:
        '''
        Return a string with the data of the Heartbeat.

        Returns:
            str: string with the data of the Heartbeat.
        '''
        return f'Heartbeat info: \nCU_ID: {self.cu_id}\nTimestamp: {self.timestamp}'

class CommDataDeviceC:
    '''
    Class used to store data of a device.
    '''

    def __init__(self, cu_id : int, comp_dev_id : int, serial_number : int,
                link_name : str|int) -> None:
        '''
        Initialize the class with the device info.

        Args:
            cu_id (int): id of the CU
            comp_dev_id (int): ID of the compatible device
            serial_number (int): serial number of the device
            link_name (str): name of the link that the device is connected
        '''
        self.cu_id = cu_id
        self.comp_dev_id = comp_dev_id
        self.serial_number = serial_number
        self.link_name = link_name

    def __str__(self) -> str:
        '''
        Return a string with the data of the Device.

        Returns:
            str: string with the data of the Device.
        '''
        return f'Device info: \nCU_ID: {self.cu_id}\nComp_dev_id: {self.comp_dev_id}\n' + \
            f'SN: {self.serial_number}\nLink name: {self.link_name}'


class CommDataMnCmdDataC:
    '''
    Class used to store data of a command sent among sevices of master node.
    '''
    def __init__(self, cmd_type : CommDataMnCmdTypeE, cu_id : int, **kwargs) -> None:
        if isinstance(cmd_type, CommDataMnCmdTypeE):
            self.cmd_type = cmd_type
            self.cu_id = cu_id
            if cmd_type is CommDataMnCmdTypeE.LAUNCH:
                if 'cs_id' in kwargs:
                    self.cs_id = kwargs['cs_id']
                else:
                    raise ValueError('Missing argument cs_id')
            elif cmd_type is CommDataMnCmdTypeE.INF_DEV:
                if 'devices' in kwargs:
                    self.devices = kwargs['devices']
                else:
                    raise ValueError('Missing argument devices')
            elif cmd_type is CommDataMnCmdTypeE.REQ_DETECT:
                pass
        else:
            raise TypeError(f'cmd_type must be of type CommDataMnCmdTypeE, not {type(cmd_type)}')

#######################            FUNCTIONS             #######################



#!/usr/bin/python3

'''
This file specifies what is going to be exported from this module.
'''

from .comm_data import (CommDataCuC, CommDataDeviceC, CommDataHeartbeatC, CommDataRegisterTypeE,
                        CommDataMnCmdTypeE, CommDataMnCmdDataC)

__all__ = [
    'CommDataCuC', 'CommDataDeviceC', 'CommDataHeartbeatC', 'CommDataRegisterTypeE',
    'CommDataMnCmdTypeE', 'CommDataMnCmdDataC'
]



#!/usr/bin/python3
'''
Definition of MID DATA experiment related info.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################
from typing import List

#######################       THIRD PARTY IMPORTS        #######################
from enum import Enum
from datetime import datetime

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
from system_logger_tool import sys_log_logger_get_module_logger
log = sys_log_logger_get_module_logger(__name__)


#######################          PROJECT IMPORTS         #######################

#######################          MODULE IMPORTS          #######################
#######################              ENUMS               #######################

class CyclerDataExpStatusE(Enum):
    '''
    Reachable states in an experiment.
    '''
    QUEUED      = "QUEUED"
    RUNNING     = "RUNNING"
    PAUSED      = "PAUSED"
    FINISHED    = "FINISHED"
    ERROR       = "ERROR"

class CyclerDataPwrModeE(Enum):
    '''
    Types of modes that can be applied to a instruction on an experiment.
    '''
    WAIT    = 0
    CV_MODE = 1
    CC_MODE = 2
    CP_MODE = 3
    DISABLE = 4

class CyclerDataPwrLimitE(Enum):
    '''
    Types of limits that can be applied to a instruction on an experiment.
    '''
    TIME    = 0
    VOLTAGE = 1
    CURRENT = 2
    POWER   = 3


#######################             CLASSES              #######################
class CyclerDataInstructionC:
    '''
    Instruction to applied on experiments.
    '''
    def __init__(self, instr_id: int| None= None, mode : CyclerDataPwrModeE| None= None,
                ref : int| None= None, limit_type : CyclerDataPwrLimitE| None= None,
                limit_ref : int| None= None):
        '''
        Initialize Instruction.

        Args:
            mode (CyclerDataPwrModeE): power mode applied (CV, CC, CP, etc)
            ref (int): reference applied on this power mode
            limit_type (CyclerDataPwrLimitE): limit that that triggers the end of
                this instruction
            limit_ref (int): limit reference to be reached to instruction completion
        '''
        self.instr_id : int| None= instr_id
        self.mode : CyclerDataPwrModeE| None = mode
        self.ref : int| None = ref
        self.limit_type : CyclerDataPwrLimitE| None = limit_type
        self.limit_ref : int| None = limit_ref


class CyclerDataExperimentC:
    '''
    Experiment relate information.
    '''
    def __init__(self, exp_id : int|None= None, name : str|None= None,
                date_begin : datetime|None= None, date_finish : datetime|None= None,
                status : CyclerDataExpStatusE|None= None):
        '''
        Initialize Experiment instance with the given parameters.

        Args:
            exp_id (int): experiment ID used to distinguish between experiments
            name (str): Name that the used given to the experiment
            date_begin (datetime): Date when the experiment started
            date_finish (datetime): Date when the experiment finished. If the
                experiment is not finished, this value is None
            status (CyclerDataExpStatusE): Current status of the experiment
        '''
        self.exp_id : int|None = exp_id
        self.name : str|None = name
        self.date_begin : datetime|None = date_begin
        self.date_finish : datetime|None = date_finish
        self.status : CyclerDataExpStatusE|None = status

class CyclerDataPwrRangeC:
    '''
    Power range that specify operational limits for voltage and current.
    '''
    def __init__(self, volt_max : int|None = None, volt_min : int|None = None,
                curr_max : int|None = None, curr_min : int|None = None):
        '''
        Initialize the class .

        Args:
            volt_max (int): [description]
            volt_min (int): [description]
            curr_max (int): [description]
            curr_min (int): [description]
        '''
        self.fill_voltage(volt_max, volt_min)
        self.fill_current(curr_max, curr_min)

    def fill_voltage(self, volt_max : int, volt_min : int) -> None:
        '''
        Fill voltage limits.

        Args:
            volt_max (int): Maximum voltage
            volt_min (int): Minimum voltage
        '''
        error_detected = True
        if volt_max is None and volt_min is None:
            error_detected = False
        elif volt_max is not None and volt_min is not None:
            if volt_max >= volt_min:
                error_detected = False
        if error_detected:
            msg = "Invalid voltage range"
            log.error(msg)
            raise ValueError(msg)

        self.__volt_max = volt_max
        self.__volt_min = volt_min

    def fill_current(self, curr_max : int, curr_min : int) -> None:
        '''
        Fill current limits.

        Args:
            curr_max (int): Maximum current
            curr_min (int): Minimum current
        '''
        error_detected = True
        if curr_max is None and curr_min is None:
            error_detected = False
        elif curr_max is not None and curr_min is not None:
            if curr_max >= curr_min:
                error_detected = False
        if error_detected:
            msg = f"Invalid current range inputs: max {curr_max} min {curr_min}"
            log.error(msg)
            raise ValueError(msg)

        self.__curr_max = curr_max
        self.__curr_min = curr_min

    @property
    def curr_max(self) -> int|None:
        '''
        Get max current.

        Returns:
            int: Max Current
        '''
        return self.__curr_max

    @property
    def curr_min(self) -> int|None:
        '''
        Get min current.

        Returns:
            int: Min Current
        '''
        return self.__curr_min

    @property
    def volt_max(self) -> int|None:
        '''
        Get max voltage.

        Returns:
            int: Max Voltage
        '''
        return self.__volt_max

    @property
    def volt_min(self) -> int|None:
        '''
        Get min voltage.

        Returns:
            int: Min Voltage
        '''
        return self.__volt_min

    def in_range_voltage(self, aux_pwr_range: CyclerDataPwrRangeC) -> bool:
        '''
        Check if the actual instance is less or equal than the other instance.

        Args:
            other (CyclerDataPwrRangeC): Instance to compare with

        Returns:
            bool: True if the voltage values are inside values of the other
                instance, False otherwise
        '''
        res = False
        if (self.volt_max is not None and aux_pwr_range.volt_max is not None and
            self.volt_max <= aux_pwr_range.volt_max and self.volt_min >= aux_pwr_range.volt_min):
            res = True
        return res

    def in_range_current(self, aux_pwr_range: CyclerDataPwrRangeC) -> bool:
        '''
        Check if the actual instance is less or equal than the other instance.

        Args:
            other (CyclerDataPwrRangeC): Instance to compare with

        Returns:
            bool: True if the current values are inside values of the other
                instance, False otherwise
        '''
        res = False
        if (self.curr_max is not None and aux_pwr_range.curr_max is not None and
            self.curr_max <= aux_pwr_range.curr_max and self.curr_min >= aux_pwr_range.curr_min):
            res = True
        return res

    def no_current(self) -> bool:
        '''
        Check if the current limits are None.
        '''
        return self.curr_max is None and self.curr_min is None

    def no_voltage(self) -> bool:
        '''
        Check if the voltage limits are None.
        '''
        return self.volt_max is None and self.volt_min is None

class CyclerDataProfileC:
    '''
    The class of CyclerDataProfileC is a class that is used for the MIDDataProfileC .
    '''
    def __init__(self, name : str|None = None, power_range : CyclerDataPwrRangeC|None = None,
            instructions : List[CyclerDataInstructionC]|None = None):
        '''
        Initialize the class .

        '''
        self.name : str|None = name
        self.instructions : List[CyclerDataInstructionC]|None = instructions
        self.range : CyclerDataPwrRangeC|None = power_range


class CyclerDataAlarmC:
    """
    Alarm triggered to notify that an unexpected event has occurred
    during cycler operation.
    """
    def __init__(self, timestamp : datetime|None = None, code : int|None = None,
                value : int|None = None):
        '''
        Initialize Alarm instance with the raising timestamp and with the
        associated alarm code. It also contains the value that triggered the alarm.

        Args:
            timestamp (datetime): Timestamp when the alarm was raised
            code (int): Code of the alarm
            value (int): Value that triggered the alarm
        '''
        self.timestamp : datetime|None = timestamp
        self.code : int|None = code
        self.value : int|None = value



#!/usr/bin/python3
'''
Definition of Cycler Data devices used on battery cycler.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################
from typing import Dict, List

#######################       THIRD PARTY IMPORTS        #######################
from enum import Enum

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
from system_logger_tool import sys_log_logger_get_module_logger
log = sys_log_logger_get_module_logger(__name__)

#######################          PROJECT IMPORTS         #######################

#######################          MODULE IMPORTS          #######################

#######################              ENUMS               #######################
class CyclerDataDeviceStatusE(Enum):
    '''
    Device status types.
    All the errors different from comm error will be interpreted as internal error.
    '''
    COMM_ERROR = -1
    OK = 0
    INTERNAL_ERROR = 1

class CyclerDataDeviceTypeE(Enum):
    '''
    Allowed power device types in the system.
    '''
    SOURCE      = "Source"
    BISOURCE    = "BiSource"
    LOAD        = "Load"
    EPC         = 'Epc'
    BMS         = "Bms"
    BK          = "Bk"
    FLOW        = "Flow"

#######################             CLASSES              #######################

class CyclerDataDeviceStatusC:
    '''Handles status of the driver power.
    '''
    def __init__(self, error: int|CyclerDataDeviceStatusE, dev_db_id: int) -> None:
        self.dev_db_id: int= dev_db_id
        if isinstance(error, CyclerDataDeviceStatusE):
            self.__status = error
            self.__error_code = error.value
        else:
            self.__error_code = error
            if error > 0:
                self.__status = CyclerDataDeviceStatusE.INTERNAL_ERROR
            else:
                self.__status = CyclerDataDeviceStatusE(error)

    def __str__(self) -> str:
        result = f"Error code: {self.__error_code} \t Status: {self.__status}"
        return result

    def __eq__(self, cmp_obj: Enum) -> bool:
        return self.__status == cmp_obj

    @property
    def error_code(self) -> int:
        '''The error code associated with this request .
        Args:
            - None
        Returns:
            - (int): The error code associated with this request.
        Raises:
            - None
        '''
        return self.__error_code

    @property
    def value(self) -> int:
        ''' Value of status.
        Args:
            - None
        Returns:
            - (int): Value of status.
        Raises:
            - None
        '''
        return self.__status.value

    @property
    def name(self) -> str:
        ''' Name of status.
        Args:
            - None
        Returns:
            - (int): name of status.
        Raises:
            - None
        '''
        return self.__status.name

class CyclerDataLinkConfC: #pylint: disable=too-many-instance-attributes
    '''
    A class method that implements the CyclerDataLinkConf class .
    '''
    # pylint: disable=too-many-arguments
    def __init__(self, **kargs) -> None:
        """Constructor of the class, the attributes will be initialized while running
        """
        accepted_attributes = {'baudrate': int, 'parity': str, 'stopbits': int , 'bytesize': int,
            'timeout':float, 'write_timeout': float, 'inter_byte_timeout':float, 'separator':str}
        for key, value in kargs.items():
            key = key.lower()
            if key in accepted_attributes:
                if not  isinstance(value, accepted_attributes[key]):
                    value = accepted_attributes[key](value)
                if key == 'parity':
                    value = value.lower()
                    if 'odd' in value:
                        value = 'O'
                    elif 'even' in value:
                        value = 'E'
                    elif 'none' in value:
                        value = 'N'
                    elif 'mark' in value:
                        value = 'M'
                    elif 'space' in value:
                        value = 'S'
                    else:
                        log.error("Wrong value for parity")
                        raise ValueError("Wrong value for parity")
                if  key == 'separator' and value =='\\n':
                    value = '\n'
                setattr(self, key, value)

class CyclerDataDeviceC: # pylint: disable=too-many-instance-attributes
    '''
    A class method that implements the CyclerDataDevice class .
    '''

    # pylint: disable=too-many-arguments
    def __init__(self, dev_db_id: int|None = None, manufacturer : str| None= None,
                model :str| None= None, serial_number : str| None= None,
                device_type : CyclerDataDeviceTypeE| None= None, iface_name : str|int| None= None,
                mapping_names : Dict| None= None,
                link_configuration: CyclerDataLinkConfC|None = None) -> None:
        """Initialize the attributes of the device .

        Args:
            manufacturer (str, optional): [description]. Defaults to None.
            model (str, optional): [description]. Defaults to None.
            serial_number (str, optional): [description]. Defaults to None.
            device_type (CyclerDataDeviceTypeE, optional): [description]. Defaults to None.
            iface_name (str, optional): [Name needed to communicate,in case of serial will be the
              port while in can will be the can id]. Defaults to None.
            mapping_names (Dict, optional): [description]. Defaults to None.
            link_configuration (CyclerDataLinkConfSerialC, optional): [description]. Defaults to
              None.
        """
        ## Check if is initialized to none
        if device_type is not None:
            device_type = CyclerDataDeviceTypeE(device_type)
        self.dev_db_id : int|None = dev_db_id
        self.manufacturer : str| None= manufacturer
        self.model : str| None = model
        self.serial_number : str| None = serial_number
        self.device_type : CyclerDataDeviceTypeE|None = device_type
        self.iface_name :str|int| None = iface_name
        self.mapping_names : Dict| None = mapping_names
        self.link_conf: CyclerDataLinkConfC|None = link_configuration

class CyclerDataCyclerStationC:
    '''
    Cycler station information.
    '''
    def __init__(self, cs_id: int| None= None, name: str| None= None,
                devices: List[CyclerDataDeviceC]| None= None, deprecated: bool|None = None):
        '''
        Initialize CyclerStation instance with the given parameters.

        Args:
            name (str): Name of the cycler station
            cs_id (str): ID of the cycler station
            devices (List[CyclerDataDeviceC]): List of devices included in the cycler station
            deprecated (bool, optional): Flag that indicates if the cycler station is deprecated
        '''
        self.name : str| None = name
        self.cs_id : int| None = cs_id
        self.devices : List[CyclerDataDeviceC]| None = devices
        self.deprecated: bool|None = deprecated



#!/usr/bin/python3
'''
Definition of MID DATA devices used on battery cycler.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations
#######################         GENERIC IMPORTS          #######################
from typing import List
#######################       THIRD PARTY IMPORTS        #######################

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
from system_logger_tool import sys_log_logger_get_module_logger
log = sys_log_logger_get_module_logger(__name__)


#######################          PROJECT IMPORTS         #######################

#######################          MODULE IMPORTS          #######################
from .cycler_data_experiment import CyclerDataPwrModeE
from .cycler_data_device import CyclerDataDeviceStatusC
#######################              ENUMS               #######################

#######################             CLASSES              #######################

class CyclerDataAllStatusC:
    '''
    Class that gather status of all used devices.
    '''

    def __init__(self):
        '''
        Intialize the instance with the given status.
        '''
        # Power device is the main device, the  status will be overwriten by the devices in use
        self.pwr_dev : CyclerDataDeviceStatusC| None = None
        self.pwr_mode : CyclerDataPwrModeE| None = None

class CyclerDataGenMeasC:
    '''
    Class used to store generic power measures.
    '''

    def __init__(self, voltage: int= 0,
                 current: int= 0, power: int= 0, instr_id: int|None = None) -> None:
        '''
        Initialize the instance with the given measures.

        Args:
            voltage (int): measured voltage in the battery
            current (int): instant current in the battery
            power (int): active power applied to the battery in a given instant
            timestamp (datetime): instante when the measures has been taken
        '''
        self.voltage : int = voltage
        self.current : int = current
        self.power : int = power
        self.instr_id : int|None = instr_id

class CyclerDataExtMeasC:
    '''
    Class used to store extended mesaures specified by the user.
    '''

    def __init__(self):
        '''
        Initialize the with the specified extended measures.
        '''

class CyclerDataMergeTagsC:
    """Class to describe which attributes should be included or excluded when mergin shared objects.
    """
    def __init__(self, status_attrs: List[str], gen_meas_attrs: List[str],
                 ext_meas_attrs: List[str]) -> None:
        self.status_attrs : List[str] = status_attrs
        self.gen_meas_attrs : List[str] = gen_meas_attrs
        self.ext_meas_attrs : List[str] = ext_meas_attrs



#!/usr/bin/python3
'''
Definition of MID DATA battery related info.
'''
#######################        MANDATORY IMPORTS         #######################
from __future__ import annotations

#######################         GENERIC IMPORTS          #######################

#######################       THIRD PARTY IMPORTS        #######################

#######################      SYSTEM ABSTRACTION IMPORTS  #######################
from system_logger_tool import sys_log_logger_get_module_logger
log = sys_log_logger_get_module_logger(__name__)
#######################          PROJECT IMPORTS         #######################

#######################          MODULE IMPORTS          #######################
from .cycler_data_experiment import CyclerDataPwrRangeC
#######################              ENUMS               #######################

#######################             CLASSES              #######################

class CyclerDataBatteryC:
    '''
    Generic battery info.
    '''
    # pylint: disable=too-many-arguments
    def __init__(self, name: str| None= None, model : str| None= None,
                 elec_ranges: CyclerDataPwrRangeC|None= None) -> None:

        self.name : str|None = name
        self.model : str|None = model
        self.elec_ranges: CyclerDataPwrRangeC|None = elec_ranges

class CyclerDataRedoxBatC(CyclerDataBatteryC):
    '''
    Redox flow specific battery info.
    '''
    # pylint: disable=too-many-arguments
    def __init__(self, electrolyte_vol : int| None= None, name: str| None= None,
                model : str| None= None, elec_ranges: CyclerDataPwrRangeC|None = None) -> None:

        self.electrolyte_vol : int| None = electrolyte_vol
        super().__init__(name= name, model= model, elec_ranges= elec_ranges)


class CyclerDataLithiumBatC(CyclerDataBatteryC):
    '''
    Lithium specific battery info.
    '''
    # pylint: disable=too-many-arguments
    def __init__(self, capacity : int| None= None, name: str| None= None, model : str| None= None,
                elec_ranges: CyclerDataPwrRangeC|None= None) -> None:

        self.capacity : int| None= capacity
        super().__init__(name= name, model= model, elec_ranges= elec_ranges)



#!/usr/bin/python3

'''
This file specifies what is going to be exported from this module.
'''

from .cycler_data_device import (CyclerDataDeviceC, CyclerDataDeviceStatusC,
                CyclerDataDeviceStatusE, CyclerDataDeviceTypeE, CyclerDataLinkConfC,
                CyclerDataCyclerStationC)
from .cycler_data_experiment import (CyclerDataPwrLimitE, CyclerDataPwrModeE, CyclerDataProfileC,
                CyclerDataAlarmC, CyclerDataPwrRangeC, CyclerDataExperimentC, CyclerDataExpStatusE,
                CyclerDataInstructionC)
from .cycler_data_common import (CyclerDataAllStatusC, CyclerDataExtMeasC, CyclerDataGenMeasC,
                                CyclerDataMergeTagsC)
from .cycler_data_battery import CyclerDataBatteryC, CyclerDataLithiumBatC, CyclerDataRedoxBatC

__all__ = [
    'CyclerDataDeviceStatusE', 'CyclerDataDeviceTypeE', 'CyclerDataDeviceStatusC',
    'CyclerDataDeviceC', 'CyclerDataLinkConfC', 'CyclerDataPwrLimitE', 'CyclerDataPwrModeE',
    'CyclerDataPwrRangeC', 'CyclerDataAlarmC', 'CyclerDataExperimentC', 'CyclerDataExpStatusE',
    'CyclerDataInstructionC', 'CyclerDataCyclerStationC', 'CyclerDataProfileC',
    'CyclerDataAllStatusC', 'CyclerDataExtMeasC', 'CyclerDataGenMeasC', 'CyclerDataBatteryC',
    'CyclerDataLithiumBatC', 'CyclerDataRedoxBatC', 'CyclerDataMergeTagsC'
]
